@@ -25,9 +25,9 @@
 
 prepanel.default.bwplot <-
     function(x, y, box.ratio,
-             horizontal = TRUE,
+             horizontal = TRUE, nlevels,
              origin = NULL, stack = FALSE,
-             levels.fos = length(unique(y)), ...)
+             ...)
 {
     ## This function needs to work for all high level functions in the
     ## bwplot family, namely bwplot, dotplot, stripplot and
@@ -35,13 +35,19 @@ prepanel.default.bwplot <-
     ## getting the ranges. For stacked barcharts, things are slightly
     ## complicated.
 
-    if (length(x) && length(y)) {
-        
+    if (length(x) && length(y))
+    {
         #if (!is.numeric(x)) x <- as.numeric(x)
         #if (!is.numeric(y)) y <- as.numeric(y)
 
         temp <- .5  #* box.ratio/(box.ratio+1)
         if (horizontal)
+        {
+            if (!is.factor(y)) ## y came from a shingle
+            {
+                if (missing(nlevels)) nlevels <- length(unique(y))
+                y <- factor(y, levels = 1:nlevels)
+            }
             list(xlim =
                  if (stack) {
                      foo1 <- if (any(x > 0)) range( by(x[x>0], y[x>0, drop = TRUE], sum)) else 0
@@ -50,15 +56,20 @@ prepanel.default.bwplot <-
                  }
                  else if (is.numeric(x)) range(x[is.finite(x)], origin)
                  else levels(x),
-                 ylim =
-                 if (is.numeric(y)) range(y[is.finite(y)])
-                 else levels(y),
-                 ##ylim = c(1-temp, levels.fos + temp),
+                 ylim = levels(y),
+                 yat = sort(unique(as.numeric(y))),
                  dx = 1,
                  dy = 1)
-        else 
-            list(xlim = if (is.numeric(x)) range(x[is.finite(x)]) else levels(x),
-                 ##xlim = c(1-temp, levels.fos + temp),
+        }
+        else
+        {
+            if (!is.factor(x)) ## x came from a shingle
+            {
+                if (missing(nlevels)) nlevels <- length(unique(x))
+                x <- factor(x, levels = 1:nlevels)
+            }
+            list(xlim = levels(x),
+                 xat = sort(unique(as.numeric(x))),
                  ylim =
                  if (stack) {
                      foo1 <- if (any(y > 0)) range( by(y[y>0], x[y>0], sum)) else 0
@@ -69,10 +80,11 @@ prepanel.default.bwplot <-
                  else levels(y),
                  dx = 1,
                  dy = 1)
+        }
     }
-    else list(c(NA, NA),
-              c(NA, NA),
-              1, 1)
+    else list(xlim = c(NA, NA),
+              ylim = c(NA, NA),
+              dx = 1, dy = 1)
 }
 
 
@@ -86,6 +98,9 @@ panel.barchart <-
              stack = FALSE,
              groups = NULL, 
              col = if (is.null(groups)) bar.fill$col else superpose.fill$col,
+             border = if (is.null(groups)) bar.fill$border else superpose.fill$border,
+             lty = if (is.null(groups)) bar.fill$lty else superpose.fill$lty,
+             lwd = if (is.null(groups)) bar.fill$lwd else superpose.fill$lwd,
              ...)
 {
     x <- as.numeric(x)
@@ -106,12 +121,15 @@ panel.barchart <-
     groupSub <- function(groups, subscripts, ...)
         groups[subscripts]
 
-    if (horizontal) {
+    if (horizontal)
+    {
 
         ## No grouping
 
-        if (is.null(groups)) {
-            if (is.null(origin)) {
+        if (is.null(groups))
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$xscale[1]
                 reference <- FALSE
             }
@@ -122,7 +140,9 @@ panel.barchart <-
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
-            grid.rect(gp = gpar(fill = col),
+            grid.rect(gp =
+                      gpar(fill = col, col = border,
+                           lty = lty, lwd = lwd),
                       y = y,
                       x = rep(origin, length(y)),
                       height = rep(height, length(y)),
@@ -133,7 +153,8 @@ panel.barchart <-
 
         ## grouped, with stacked bars
 
-        else if (stack) {
+        else if (stack)
+        {
 
             if (!is.null(origin) && origin != 0)
                 warning("origin forced to 0 for stacked bars")
@@ -142,7 +163,11 @@ panel.barchart <-
             vals <- sort(unique(groups))
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
+
             col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
 
             height <- box.ratio/(1 + box.ratio)
 
@@ -152,13 +177,18 @@ panel.barchart <-
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
 
-            for (i in unique(y)) {
+            for (i in unique(y))
+            {
                 ok <- y == i
                 ord <- sort.list(groups[ok])
                 pos <- x[ok][ord] > 0
                 nok <- sum(pos)
                 if (nok > 0)
-                    grid.rect(gp = gpar(fill = col[groups[ok][ord][pos]]),
+                    grid.rect(gp =
+                              gpar(fill = col[groups[ok][ord][pos]],
+                                   col = border[groups[ok][ord][pos]],
+                                   lty = lty[groups[ok][ord][pos]],
+                                   lwd = lwd[groups[ok][ord][pos]]),
                               y = rep(i, nok),
                               x = cumsum(c(0, x[ok][ord][pos][-nok])),
                               height = rep(height, nok),
@@ -168,7 +198,11 @@ panel.barchart <-
                 neg <- x[ok][ord] < 0
                 nok <- sum(neg)
                 if (nok > 0)
-                    grid.rect(gp = gpar(fill = col[groups[ok][ord][neg]]),
+                    grid.rect(gp =
+                              gpar(fill = col[groups[ok][ord][neg]],
+                                   col = border[groups[ok][ord][neg]],
+                                   lty = lty[groups[ok][ord][neg]],
+                                   lwd = lwd[groups[ok][ord][neg]]),
                               y = rep(i, nok),
                               x = cumsum(c(0, x[ok][ord][neg][-nok])),
                               height = rep(height, nok),
@@ -180,8 +214,10 @@ panel.barchart <-
 
         ## grouped, with side by side bars
 
-        else {
-            if (is.null(origin)) {
+        else
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$xscale[1]
                 reference <- FALSE
             }
@@ -189,7 +225,11 @@ panel.barchart <-
             vals <- sort(unique(groups))
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
+
             col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
 
             height <- box.ratio/(1 + nvals * box.ratio)
             if (reference)
@@ -197,10 +237,15 @@ panel.barchart <-
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
-            for (i in unique(y)) {
+            for (i in unique(y))
+            {
                 ok <- y == i
                 nok <- sum(ok)
-                grid.rect(gp = gpar(fill = col[groups[ok]]),
+                grid.rect(gp =
+                          gpar(fill = col[groups[ok]],
+                               col = border[groups[ok]],
+                               lty = lty[groups[ok]],
+                               lwd = lwd[groups[ok]]),
                           y = (i + height * (groups[ok] - (nvals + 1)/2)),
                           x = rep(origin, nok), 
                           height = rep(height, nok),
@@ -213,9 +258,12 @@ panel.barchart <-
     
     ## if not horizontal
 
-    else {
-        if (is.null(groups)) {
-            if (is.null(origin)) {
+    else
+    {
+        if (is.null(groups))
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$yscale[1]
                 reference <- FALSE
             }
@@ -227,7 +275,9 @@ panel.barchart <-
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
 
-            grid.rect(gp = gpar(fill = col),
+            grid.rect(gp =
+                      gpar(fill = col, col = border,
+                           lty = lty, lwd = lwd),
                       x = x,
                       y = rep(origin, length(x)),
                       width = rep(width, length(x)),
@@ -235,7 +285,8 @@ panel.barchart <-
                       just = c("centre", "bottom"),
                       default.units = "native")
         }
-        else if (stack) {
+        else if (stack)
+        {
 
             if (!is.null(origin) && origin != 0)
                 warning("origin forced to 0 for stacked bars")
@@ -245,12 +296,11 @@ panel.barchart <-
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
 
-            regions <- trellis.par.get("regions")
-            numcol.r <- length(col)
-            col <- 
-                if (numcol.r <= nvals) rep(col, length = nvals)
-                else col[floor(1+(vals-1)*(numcol.r-1)/(nvals-1))]
-            
+            col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
+
             width <- box.ratio/(1 + box.ratio)
 
             if (reference)
@@ -259,13 +309,18 @@ panel.barchart <-
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
 
-            for (i in unique(x)) {
+            for (i in unique(x))
+            {
                 ok <- x == i
                 ord <- sort.list(groups[ok])
                 pos <- y[ok][ord] > 0
                 nok <- sum(pos)
                 if (nok > 0)
-                    grid.rect(gp = gpar(fill = col[groups[ok][ord][pos]]),
+                    grid.rect(gp =
+                              gpar(fill = col[groups[ok][ord][pos]],
+                                   col = border[groups[ok][ord][pos]],
+                                   lty = lty[groups[ok][ord][pos]],
+                                   lwd = lwd[groups[ok][ord][pos]]),
                               x = rep(i, nok),
                               y = cumsum(c(0, y[ok][ord][pos][-nok])),
                               width = rep(width, nok),
@@ -275,7 +330,11 @@ panel.barchart <-
                 neg <- y[ok][ord] < 0
                 nok <- sum(neg)
                 if (nok > 0)
-                    grid.rect(gp = gpar(fill = col[groups[ok][ord][neg]]),
+                    grid.rect(gp =
+                              gpar(fill = col[groups[ok][ord][neg]],
+                                   col = border[groups[ok][ord][neg]],
+                                   lty = lty[groups[ok][ord][neg]],
+                                   lwd = lwd[groups[ok][ord][neg]]),
                               x = rep(i, nok),
                               y = cumsum(c(0, y[ok][ord][neg][-nok])),
                               width = rep(width, nok),
@@ -286,8 +345,10 @@ panel.barchart <-
 
             
         }
-        else {
-            if (is.null(origin)) {
+        else
+        {
+            if (is.null(origin))
+            {
                 origin <- current.viewport()$yscale[1]
                 reference = FALSE
             }
@@ -296,22 +357,26 @@ panel.barchart <-
             nvals <- length(vals)
             groups <- groupSub(groups, ...)
 
-            regions <- trellis.par.get("regions")
-            numcol.r <- length(col)
-            col <- 
-                if (numcol.r <= nvals) rep(col, length = nvals)
-                else col[floor(1+(vals-1)*(numcol.r-1)/(nvals-1))]
-            
+            col <- rep(col, length = nvals)
+            border <- rep(border, length = nvals)
+            lty <- rep(lty, length = nvals)
+            lwd <- rep(lwd, length = nvals)
+
             width <- box.ratio/(1 + nvals * box.ratio)
             if (reference)
                 panel.abline(h = origin,
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd)
-            for (i in unique(x)) {
+            for (i in unique(x))
+            {
                 ok <- x == i
                 nok <- sum(ok)
-                grid.rect(gp = gpar(fill = col[groups[ok]]),
+                grid.rect(gp =
+                          gpar(fill = col[groups[ok]],
+                               col = border[groups[ok]],
+                               lty = lty[groups[ok]],
+                               lwd = lwd[groups[ok]]),
                           x = (i + width * (groups[ok] - (nvals + 1)/2)),
                           y = rep(origin, nok), 
                           width = rep(width, nok),
@@ -325,7 +390,6 @@ panel.barchart <-
 
 
 
-
 panel.dotplot <-
     function(x, y, horizontal = TRUE,
              pch = if (is.null(groups)) dot.symbol$pch else sup.symbol$pch,
@@ -333,7 +397,7 @@ panel.dotplot <-
              lty = dot.line$lty,
              lwd = dot.line$lwd,
              col.line = dot.line$col,
-             levels.fos = NULL,
+             levels.fos = if (horizontal) unique(y) else unique(x),
              groups = NULL,
              ...)
 {
@@ -344,33 +408,35 @@ panel.dotplot <-
     dot.symbol <- trellis.par.get("dot.symbol")
     sup.symbol <- trellis.par.get("superpose.symbol")
 
-    if (horizontal) {
+    if (horizontal)
+    {
         yscale <- current.viewport()$yscale
-        if (is.null(levels.fos))
-            levels.fos <- floor(yscale[2])-ceiling(yscale[1])+1
-        panel.abline(h=1:levels.fos, col=col.line,
-                     lty=lty, lwd=lwd)
+        panel.abline(h = levels.fos,
+                     col = col.line,
+                     lty = lty, lwd = lwd)
         if (is.null(groups)) 
-            panel.xyplot(x = x, y = y, col = col, pch = pch, lty = lty, lwd = lwd, 
+            panel.xyplot(x = x, y = y, col = col,
+                         pch = pch, lty = lty, lwd = lwd, 
                          horizontal = horizontal, ...)
         else
             panel.superpose(x = x, y = y, groups = groups,
-                            col = col, pch = pch, lty = lty, lwd = lwd, 
+                            col = col, pch = pch,
+                            lty = lty, lwd = lwd, 
                             horizontal = horizontal, ...)
     }
-    else {
+    else
+    {
         xscale <- current.viewport()$xscale
-        if (is.null(levels.fos))
-            levels.fos <- floor(xscale[2])-ceiling(xscale[1])+1
-        panel.abline(v=1:levels.fos, col=col.line,
-                     lty=lty, lwd=lwd)
+        panel.abline(v = levels.fos, col = col.line,
+                     lty = lty, lwd = lwd)
         if (is.null(groups)) 
             panel.xyplot(x = x, y = y, col = col, pch = pch,
                          lty = lty, lwd = lwd, 
                          horizontal = horizontal, ...)
         else 
             panel.superpose(x = x, y = y, groups = groups,
-                            col = col, pch = pch, lty = lty, lwd = lwd, 
+                            col = col, pch = pch,
+                            lty = lty, lwd = lwd, 
                             horizontal = horizontal, ...)
     }
 }
@@ -401,13 +467,15 @@ panel.stripplot <-
 
 
 
-
 panel.bwplot <-
     function(x, y, box.ratio=1, horizontal = TRUE, pch=box.dot$pch,
              col = box.dot$col, cex = box.dot$cex,
-             font = box.dot$font, fontfamily = box.dot$fontfamily, fontface = box.dot$fontface, 
+             font = box.dot$font,
+             fontfamily = box.dot$fontfamily,
+             fontface = box.dot$fontface, 
              fill = box.rectangle$fill, varwidth = FALSE,
-             levels.fos = NULL, coef = 1.5, ...)
+             levels.fos = if (horizontal) unique(y) else unique(x),
+             coef = 1.5, ...)
 {
     x <- as.numeric(x)
     y <- as.numeric(y)
@@ -425,159 +493,159 @@ panel.bwplot <-
     ## be working on the premise that EACH INTEGER in the y-RANGE is a
     ## potential location of a boxplot. 
 
-    if (horizontal) {
+    if (horizontal)
+    {
 
         maxn <- max(by(x, y, length)) ## used if varwidth = TRUE
 
         yscale <- current.viewport()$yscale
-        if (is.null(levels.fos)) levels.fos <- floor(yscale[2])-ceiling(yscale[1])+1
         lower <- ceiling(yscale[1])
         height <- box.ratio/(1+box.ratio)
         xscale <- current.viewport()$xscale
-        if (levels.fos > 0)
-            for (i in 1:levels.fos) {
 
-                yval  <- i
-                stats <- boxplot.stats(x[y==yval], coef = coef)
+        for (yval in levels.fos)
+        {
+
+            ## yval  <- i
+            stats <- boxplot.stats(x[y==yval], coef = coef)
+            
+            if (stats$n>0)
+            {
+                pushViewport(viewport(y=unit(yval, "native"),
+                                      height = unit((if (varwidth)
+                                      sqrt(stats$n/maxn)  else 1) * height, "native"),
+                                      xscale = xscale))
                 
+                r.x <- (stats$stats[2]+stats$stats[4])/2
+                r.w <- stats$stats[4]-stats$stats[2]
+                grid.rect(x = unit(r.x, "native"), width = unit(r.w, "native"),
+                          gp = gpar(lwd = box.rectangle$lwd,
+                          lty = box.rectangle$lty,
+                          fill = fill,
+                          col = box.rectangle$col))
                 
-                if (stats$n>0)
-                {
-                    pushViewport(viewport(y=unit(yval, "native"),
-                                           height = unit((if (varwidth)
-                                           sqrt(stats$n/maxn)  else 1) * height, "native"),
-                                           xscale = xscale))
-                    
-                    r.x <- (stats$stats[2]+stats$stats[4])/2
-                    r.w <- stats$stats[4]-stats$stats[2]
-                    grid.rect(x = unit(r.x, "native"), width = unit(r.w, "native"),
-                              gp = gpar(lwd = box.rectangle$lwd,
-                              lty = box.rectangle$lty,
-                              fill = fill,
-                              col = box.rectangle$col))
+                grid.lines(x = unit(stats$stats[1:2],"native"),
+                           y=unit(c(.5,.5), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
                 
-                    grid.lines(x = unit(stats$stats[1:2],"native"),
-                               y=unit(c(.5,.5), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.lines(x = unit(stats$stats[4:5],"native"),
-                               y=unit(c(.5,.5), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.lines(x = unit(rep(stats$stats[1],2),"native"),
-                               y=unit(c(0,1), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.lines(x = unit(rep(stats$stats[5],2),"native"),
-                               y=unit(c(0,1), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.points(x = stats$stats[3], y = .5, pch = pch, 
+                grid.lines(x = unit(stats$stats[4:5],"native"),
+                           y=unit(c(.5,.5), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
+                
+                grid.lines(x = unit(rep(stats$stats[1],2),"native"),
+                           y=unit(c(0,1), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
+                
+                grid.lines(x = unit(rep(stats$stats[5],2),"native"),
+                           y=unit(c(0,1), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
+                
+                grid.points(x = stats$stats[3], y = .5, pch = pch, 
+                            gp =
+                            gpar(col = col, cex = cex,
+                                 fontfamily = fontfamily,
+                                 fontface = chooseFace(fontface, font),
+                                 fontsize = fontsize.points))
+                
+                if ((l<-length(stats$out))>0)
+                    grid.points(x = stats$out, y = rep(.5,l),
+                                pch = plot.symbol$pch,
                                 gp =
-                                gpar(col = col, cex = cex,
-                                     fontfamily = fontfamily,
-                                     fontface = chooseFace(fontface, font),
+                                gpar(col = plot.symbol$col,
+                                     cex = plot.symbol$cex,
+                                     fontfamily = plot.symbol$fontfamily,
+                                     fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
                                      fontsize = fontsize.points))
-                    
-                    if ((l<-length(stats$out))>0)
-                        grid.points(x = stats$out, y = rep(.5,l),
-                                    pch = plot.symbol$pch,
-                                    gp =
-                                    gpar(col = plot.symbol$col,
-                                         cex = plot.symbol$cex,
-                                         fontfamily = plot.symbol$fontfamily,
-                                         fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
-                                         fontsize = fontsize.points))
-                    
-                    popViewport()
                 
-                }
+                popViewport()
+                
             }
-    
+        }
+        
     }
-    else {
+    else
+    {
 
         maxn <- max(by(y, x, length)) ## used if varwidth = TRUE
 
         xscale <- current.viewport()$xscale
-        if (is.null(levels.fos)) levels.fos <- floor(xscale[2])-ceiling(xscale[1])+1
         lower <- ceiling(xscale[1])
         width <- box.ratio/(1+box.ratio)
         yscale <- current.viewport()$yscale
-        if (levels.fos > 0)
-            for (i in 1:levels.fos) {
-                xval  <- i
-                stats <- boxplot.stats(y[x==xval], coef = coef)
 
-                if (stats$n>0)
-                {
-                    pushViewport(viewport(x = unit(xval, "native"),
-                                           width = unit((if (varwidth)
-                                           sqrt(stats$n/maxn)  else 1) * width, "native"),
-                                           yscale = yscale))
-                    
-                    r.x <- (stats$stats[2]+stats$stats[4])/2
-                    r.w <- stats$stats[4]-stats$stats[2]
-                    grid.rect(y = unit(r.x, "native"), height = unit(r.w, "native"),
-                              gp = gpar(lwd = box.rectangle$lwd,
-                              lty = box.rectangle$lty,
-                              fill = fill,
-                              col = box.rectangle$col))
+        for (xval in levels.fos) {
+            ##xval  <- i
+            stats <- boxplot.stats(y[x==xval], coef = coef)
+
+            if (stats$n>0)
+            {
+                pushViewport(viewport(x = unit(xval, "native"),
+                                      width = unit((if (varwidth)
+                                      sqrt(stats$n/maxn)  else 1) * width, "native"),
+                                      yscale = yscale))
                 
-                    grid.lines(y = unit(stats$stats[1:2],"native"),
-                               x = unit(c(.5,.5), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.lines(y = unit(stats$stats[4:5],"native"),
-                               x = unit(c(.5,.5), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.lines(y = unit(rep(stats$stats[1],2),"native"),
-                               x = unit(c(0,1), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.lines(y = unit(rep(stats$stats[5],2),"native"),
-                               x = unit(c(0,1), "npc"),
-                               gp = gpar(col = box.umbrella$col,
-                               lwd = box.umbrella$lwd,
-                               lty = box.umbrella$lty))
-                    
-                    grid.points(y = stats$stats[3], x = .5, pch = pch, 
+                r.x <- (stats$stats[2]+stats$stats[4])/2
+                r.w <- stats$stats[4]-stats$stats[2]
+                grid.rect(y = unit(r.x, "native"), height = unit(r.w, "native"),
+                          gp = gpar(lwd = box.rectangle$lwd,
+                          lty = box.rectangle$lty,
+                          fill = fill,
+                          col = box.rectangle$col))
+                
+                grid.lines(y = unit(stats$stats[1:2],"native"),
+                           x = unit(c(.5,.5), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
+                
+                grid.lines(y = unit(stats$stats[4:5],"native"),
+                           x = unit(c(.5,.5), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
+                
+                grid.lines(y = unit(rep(stats$stats[1],2),"native"),
+                           x = unit(c(0,1), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
+                
+                grid.lines(y = unit(rep(stats$stats[5],2),"native"),
+                           x = unit(c(0,1), "npc"),
+                           gp = gpar(col = box.umbrella$col,
+                           lwd = box.umbrella$lwd,
+                           lty = box.umbrella$lty))
+                
+                grid.points(y = stats$stats[3], x = .5, pch = pch, 
+                            gp =
+                            gpar(col = col, cex = cex,
+                                 fontfamily = fontfamily,
+                                 fontface = chooseFace(fontface, font),
+                                 fontsize = fontsize.points))
+                
+                if ((l<-length(stats$out))>0)
+                    grid.points(y = stats$out, x = rep(.5,l),
+                                pch = plot.symbol$pch,
                                 gp =
-                                gpar(col = col, cex = cex,
-                                     fontfamily = fontfamily,
-                                     fontface = chooseFace(fontface, font),
+                                gpar(col = plot.symbol$col,
+                                     cex = plot.symbol$cex,
+                                     fontfamily = plot.symbol$fontfamily,
+                                     fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
                                      fontsize = fontsize.points))
-                    
-                    if ((l<-length(stats$out))>0)
-                        grid.points(y = stats$out, x = rep(.5,l),
-                                    pch = plot.symbol$pch,
-                                    gp =
-                                    gpar(col = plot.symbol$col,
-                                         cex = plot.symbol$cex,
-                                         fontfamily = plot.symbol$fontfamily,
-                                         fontface = chooseFace(plot.symbol$fontface, plot.symbol$font),
-                                         fontsize = fontsize.points))
-                    
-                    popViewport()
                 
-                }
+                popViewport()
+                
             }
-    
+        }
+        
     }
 }
 
@@ -721,8 +789,11 @@ bwplot <-
              ylim,
              box.ratio = 1,
              horizontal = NULL,
-             drop.unused.levels = TRUE,
+             drop.unused.levels = lattice.getOption("drop.unused.levels"),
              ...,
+             default.scales =
+             if (horizontal) list(y = list(tck = 0, alternating = FALSE, rot = 0))
+             else list(x = list(tck = 0, alternating = FALSE)),
              subscripts = !is.null(groups),
              subset = TRUE)
 {
@@ -784,8 +855,10 @@ bwplot <-
     }
 
 
-    if (horizontal) {
-        if (!(is.numeric(x))) {
+    if (horizontal)
+    {
+        if (!(is.numeric(x)))
+        {
             warning("x should be numeric")
         }
         y <- as.factorOrShingle(y)
@@ -795,8 +868,10 @@ bwplot <-
         if (missing(xlab)) xlab <- form$right.name
         if (missing(ylab)) ylab <- if (is.f.y) NULL else form$left.name
     }
-    else {
-        if (!(is.numeric(y))) {
+    else
+    {
+        if (!(is.numeric(y)))
+        {
             warning("y should be numeric")
         }
         x <- as.factorOrShingle(x)
@@ -833,18 +908,19 @@ bwplot <-
 
     ## The following is to make the default alternating FALSE for factors
     if (is.character(scales)) scales <- list(relation = scales)
-    if (is.null(scales$alternating)) {
-        if (horizontal) {
-            if (is.null(scales$y)) scales$y <- list(alternating = FALSE)
-            else if (is.null(scales$y$alternating)) scales$y$alternating <- FALSE
-        ## bug if y="free" ? but who cares
-        }
-        else {
-            if (is.null(scales$x)) scales$x <- list(alternating = FALSE)
-            else if (is.null(scales$x$alternating)) scales$x$alternating <- FALSE
-        ## bug if x="free" ? but who cares
-        }
-    }
+#     if (is.null(scales$alternating)) {
+#         if (horizontal) {
+#             if (is.null(scales$y)) scales$y <- list(alternating = FALSE)
+#             else if (is.null(scales$y$alternating)) scales$y$alternating <- FALSE
+#         ## bug if y="free" ? but who cares
+#         }
+#         else {
+#             if (is.null(scales$x)) scales$x <- list(alternating = FALSE)
+#             else if (is.null(scales$x$alternating)) scales$x$alternating <- FALSE
+#         ## bug if x="free" ? but who cares
+#         }
+#     }
+    scales <- updateList(default.scales, scales)
     foo <- c(foo,
              do.call("construct.scales", scales))
 
@@ -903,11 +979,19 @@ bwplot <-
     foo$panel.args.common <- dots
     foo$panel.args.common$box.ratio <- box.ratio
     foo$panel.args.common$horizontal <- horizontal
-    foo$panel.args.common$levels.fos <- ## fos - the factor/shingle in x/y
-        if (horizontal) num.l.y else num.l.x
     if (subscripts) foo$panel.args.common$groups <- groups
 
-
+    ## only used if shingle, important if some levels are missing
+    if (horizontal)
+    {
+        if (!is.f.y) ## y shingle
+            foo$panel.args.common$nlevels <- num.l.y
+    }
+    else
+    {
+        if (!is.f.x) ## x shingle
+            foo$panel.args.common$nlevels <- num.l.x
+    }
 
     nplots <- prod(cond.max.level)
     if (nplots != prod(sapply(foo$condlevels, length))) stop("mismatch")
@@ -933,8 +1017,10 @@ bwplot <-
             else (as.numeric(var) == cond.current.level[i])
         }
 
-        if (horizontal) {
-            if (is.f.y) {
+        if (horizontal)
+        {
+            if (is.f.y)
+            {
                 foo$panel.args[[panel.number]] <-
                     list(x = x[id],
                          ##y = as.numeric(y[id]))
@@ -943,11 +1029,13 @@ bwplot <-
                     foo$panel.args[[panel.number]]$subscripts <-
                         subscr[id]
             }
-            else {  # shingle
+            else  # shingle
+            {
                 panel.x <- numeric(0)
                 panel.y <- numeric(0)
                 if (subscripts) panel.subscr <- numeric(0)
-                for (k in 1:num.l.y) {
+                for (k in seq(length = num.l.y))
+                {
                     tid <- id & (y >= levels(y)[[k]][1]) & (y <= levels(y)[[k]][2])
                     panel.x <- c(panel.x, x[tid])
                     panel.y <- c(panel.y, rep(k,length(tid[tid])))
@@ -962,8 +1050,10 @@ bwplot <-
 
             }
         }
-        else {
-            if (is.f.x) {
+        else
+        {
+            if (is.f.x)
+            {
                 foo$panel.args[[panel.number]] <-
                     ##list(x = as.numeric(x[id]),
                     list(x = x[id],
@@ -972,11 +1062,13 @@ bwplot <-
                     foo$panel.args[[panel.number]]$subscripts <-
                         subscr[id]
             }
-            else {  # shingle
+            else   # shingle
+            {
                 panel.x <- numeric(0)
                 panel.y <- numeric(0)
                 if (subscripts) panel.subscr <- numeric(0)
-                for (k in 1:num.l.x) {
+                for (k in seq(length = num.l.x))
+                {
                     tid <- id & (x >= levels(x)[[k]][1]) & (x <= levels(x)[[k]][2])
                     panel.y <- c(panel.y, y[tid])
                     panel.x <- c(panel.x, rep(k,length(tid[tid])))
